#include "stm32f4xx.h"
#include "stm32f4xx_i2c.h"

#include "LED7Segment.h"
#include "port407v.h"


static const uint8_t lookup[] = {
		0x3F, // 0
		0x06, // 1
		0x5B, // 2
		0x4F, // 3
		0x66, // 4
		0x6D, // 5
		0x7D, // 6
		0x07, // 7
		0x7F, // 8
		0x6F, // 9
		0x77, // a
		0x7C, // b
		0x39, // C
		0x5E, // d
		0x79, // E
		0x71, // F
};

HT16K33_Display::HT16K33_Display(I2C_TypeDef *port, uint8_t i2c_addr) {
	i2c = port;
	addr = i2c_addr;
}

uint8_t HT16K33_Display::waitSR1Flags(uint32_t flags) {
	uint32_t timeout = HSE_VALUE;
	while (((i2c->SR1) & flags) != flags) {
		if (!(timeout--)) {
			panic(flags, "I2C Error\n");
			return 1;
		}
	}

	return 0;
}

uint8_t HT16K33_Display::waitUntilIdle() {
	uint32_t timeout = HSE_VALUE;

	while((i2c->SR2) & I2C_SR2_BUSY) {
		if (!(timeout--)) {
			panic(I2C_SR2_BUSY, "I2C Error");
			return 1;
		}
	}

	return 0;
}

uint8_t HT16K33_Display::start() {
	i2c->CR1 |= I2C_CR1_START;
	waitSR1Flags(I2C_SR1_SB);
	return sendAddr(I2C_Direction_Transmitter);
}

void HT16K33_Display::stop() {
    i2c->CR1 |= I2C_CR1_STOP;
    waitUntilIdle();
}

uint8_t HT16K33_Display::sendAddr(uint8_t dir) {
	uint8_t val;
	i2c->DR = (addr << 1) | dir;

	val = waitSR1Flags(I2C_SR1_ADDR | I2C_SR1_TXE);
	(void) i2c->SR2;

	return val;
}

void HT16K33_Display::sendByte(uint8_t byte) {
	i2c->DR = byte;

	waitSR1Flags(I2C_SR1_TXE);
}

void HT16K33_Display::displayCmd(bool on) {
	start();

	if (on) {
		sendByte(0x21);
	} else {
		sendByte(0x20);
	}

	stop();
}

void HT16K33_Display::init() {
	displayCmd(true);
	blinkRate(HT16K33_BLINK_OFF);
	setBrightness(15);
}

void HT16K33_Display::blinkRate(uint8_t rate) {
	if (rate > 3) {
		rate = 0;
	}

	start();
	sendByte(HT16K33_BLINK_CMD | HT16K33_BLINK_DISPLAYON | (rate << 1));
	stop();
}

void HT16K33_Display::displayOn(bool on) {
	start();
	if (on) {
		sendByte(HT16K33_BLINK_CMD | HT16K33_BLINK_DISPLAYOFF);
	} else {
		sendByte(HT16K33_BLINK_CMD | HT16K33_BLINK_DISPLAYON);
	}
}

void HT16K33_Display::setBrightness(uint8_t brightness) {
	if (brightness > 15) {
		brightness = 15;
	}

	start();
	sendByte(HT16K33_BRIGHTNESS_CMD | brightness);
	stop();
}

void HT16K33_Display::writeBuffer() {
	start();
	sendByte(0);
	for(uint8_t i= 0; i < 8; i++) {
		sendByte(displayBuffer[i] & 0xFF);
		sendByte(displayBuffer[i] >> 8);
	}

	stop();
}

void HT16K33_Display::clear() {
    for (uint8_t i = 0; i < 8; i++) {
    	if (i != 2) {
    		displayBuffer[i] = 0;
    	}
    }
}

void HT16K33_Display::print(uint16_t value) {
    clear();

    uint16_t orig = value;

    if (orig >= 1000) {
    	displayBuffer[0] = lookup[value / 1000];
    	value %= 1000;
    }

    if (orig >= 100) {
    	displayBuffer[1] = lookup[value / 100];
    	value %= 100;
    }

    if (orig >= 10) {
    	displayBuffer[3] = lookup[value / 10];
    	value %= 10;
    }

    displayBuffer[4] = lookup[value];

    writeBuffer();
}

void HT16K33_Display::error() {
	clear();

	for (uint8_t i = 0; i < 5; i++) {
		displayBuffer[i] = 0x40;
	}

	blinkRate(HT16K33_BLINK_2HZ);
	writeBuffer();
}

void HT16K33_Display::colon(bool on) {
	if (on) {
		displayBuffer[2] = 0xFF;
	} else {
		displayBuffer[2] = 0;
	}
}

void HT16K33_Display::oil() {
	blinkRate(HT16K33_BLINK_2HZ);
	clear();
	iisplayBuffer[0] = 0x3F;
	displayBuffer[1] = 0x30;
	displayBuffer[2] = 0x0;
	displayBuffer[3] = 0x38;

	writeBuffer();
}

void HT16K33_Display::fuel() {
	blinkRate(HT16K33_BLINK_2HZ);
	clear();
	displayBuffer[0] = 0x71;
	displayBuffer[1] = 0x3E;
	displayBuffer[2] = 0x0;
	displayBuffer[3] = 0x79;
	displayBuffer[4] = 0x38;

	writeBuffer();
}
